import {
    IsEnum,
    IsInt,
    IsNotEmpty,
    IsObject
} from "class-validator";
import { Permission } from '../entities/Permission';
import { PermissionAction } from '../enums/PermissionAction';
import { PermissionTarget } from '../enums/PermissionTargets';
import { ResponsePrincipal } from './ResponsePrincipal';

/**
 * Defines a track of given length.
*/
export class ResponsePermission {
    /**
     * Autogenerated unique id (primary key).
     */
    @IsInt()
    id: number;;

    /**
     * The permissions's principal.
     */
    @IsObject()
    @IsNotEmpty()
    principal: ResponsePrincipal;

    /**
     * The permissions's target.
     */
    @IsNotEmpty()
    @IsEnum(PermissionTarget)
    target: PermissionTarget;

    /**
     * The permissions's action.
     */
    @IsNotEmpty()
    @IsEnum(PermissionAction)
    action: PermissionAction;

    public constructor(permission: Permission) {
        this.id = permission.id;
        this.principal = permission.principal.toResponse();
        this.target = permission.target;
        this.action = permission.action;
    }
}
