import { validationMetadatasToSchemas } from 'class-validator-jsonschema';
import consola from "consola";
import fs from "fs";
import "reflect-metadata";
import { createExpressServer, getMetadataArgsStorage } from "routing-controllers";
import { routingControllersToSpec } from 'routing-controllers-openapi';
import authchecker from "./authchecker";
import { config } from './config';
import { ErrorHandler } from './middlewares/ErrorHandler';

const CONTROLLERS_FILE_EXTENSION = process.env.NODE_ENV === 'production' ? 'js' : 'ts';
createExpressServer({
    authorizationChecker: authchecker,
    middlewares: [ErrorHandler],
    development: config.development,
    cors: true,
    routePrefix: "/api",
    controllers: [`${__dirname}/controllers/*.${CONTROLLERS_FILE_EXTENSION}`],
});

const storage = getMetadataArgsStorage();
const schemas = validationMetadatasToSchemas({
    refPointerPrefix: "#/components/schemas/",
});

//Spec creation based on the previously created schemas
const spec = routingControllersToSpec(
    storage,
    {
        routePrefix: "/api"
    },
    {
        components: {
            schemas,
            "securitySchemes": {
                "AuthToken": {
                    "type": "http",
                    "scheme": "bearer",
                    "bearerFormat": "JWT"
                }
            }
        },
        info: {
            description: "The the backend API for the LfK! runner system.",
            title: "LfK! Backend API",
            version: "1.0.0",
        },
    }
);

try {
    fs.writeFileSync("./openapi.json", JSON.stringify(spec), { encoding: "utf-8" });
    consola.success("Exported openapi spec to openapi.json");
} catch (error) {
    consola.error("Couldn't export the openapi spec");
}
